# -*- coding: utf-8 -*-
"""
Created on Wed Aug  3 21:12:23 2022

@author: Martín Carlos Araya <martinaraya@gmail.com>

helper functions for units modules
"""

from ._isnumber import isnumber
from ._caster import tonumber